/*
 *	@(#)$Id: Informix.h,v 50.1 1997/01/12 17:54:43 johnl Exp $ 
 *
 *	$Derived-From: Informix.h,v 1.3 1995/05/25 21:18:56 timbo Archaic $
 *
 * Portions Copyright (c) 1994,1995 Tim Bunce
 * Portions Copyright (c) 1995,1996 Alligator Descartes
 * Portions Copyright (c) 1996,1997 Jonathan Leffler
 *
 * You may distribute under the terms of either the GNU General Public
 * License or the Artistic License, as specified in the Perl README file.
 */

#ifndef INFORMIX_H
#define INFORMIX_H

#include <DBIXS.h>		/* installed by the DBI module	*/
#include "esqlc.h"		/* Prototypes for ESQL/C version 5.0x etc */
#include "esqlperl.h"
#include "esqlvrsn.h"	/* Defines ESQLC_VERSION_STRING */
#include "dbdimp.h"		/* Informix implementation details */

#endif /* INFORMIX_H */
